outputs = {
"vc": False,                  # Vegetation Cover [-]
"lai": False,                 # Leaf Area Index [-]
"lai_eff": False,             # Effective Leaf Area Index [-]
"sf_soil": False,             # Soil Fraction [-]  
"lat": False,                 # Latitude [degrees]
"slope": False,               # Slope [rad]
"aspect": False,              # Aspect []
"ra_24_toa": False,           # Daily Solar Radiation At The Top Of Atmosphere [Wm-2]
"ws": False,                  # Sunset Hour Angle [Rad]
"diffusion_index": False,     # Diffusion Index 
"ra_24": False,              # Daily Solar Radiation For A Flat Surface [Wm-2]
"ra_24_toa_flat": False,      #
"stress_rad": False,          # Stress Factor For Radiation [-]
"p_air_24": False,            # Daily Air Pressure [mbar]
"vp_24": False,               # Daily Vapour Pressure [mbar]
"svp_24": False,              # Daily Slope Of Saturated Vapour Pressure Curve [mbar K-1]
"vpd_24": False,              # Daily Vapour Pressure Deficit [mbar]
"stress_vpd": False,          # Stress Factor For Vapour Pressure Deficit [-]
"stress_temp": False,         # Stress Factor For Air Temperature [-]   
"r_canopy_0": False,          # Atmospheric Canopy Resistance [sm-1]
"t_air_k_24": False,          # Daily Air Temperature [Kelvin]
"l_net": False,               # Daily Net Longwave Radiation [wm-2]
"int_mm": True,              # Interception [mm day-1]
"lh_24": False,               # Daily Latent Heat For Evaporation [j kg-1]
"int_wm2": False,            # Interception [Wm-2]
"rn_24": False,               # Net Radiation [Wm-2]      
"rn_24_canopy": False,        # Net Radiation For The Canopy [Wm-2]          
"t_air_k_i": False,           # Instantaneous Air Tempearture [K]
"vp_i": False,                # Instantaneous Vapour Pressure [mbar]
"ad_moist_i": False,          # Instantaneous Moist Air Density [kg m-3]
"ad_dry_i": False,            # Instantaneous Dry Air Density [kg m-3]
"ad_i": False,                # Instantaneous Air Density [kg m-3]
"u_b_i_bare": False,          # Instantaneous Wind Speed At Blending Height For Bare Soil [m s-1]
"lon": False,                 # longitude [degrees]
"ha": False,                  # Solar Hour Angel [rad]
# ied": False,                 # Inverse Earth Sun Distance [A U-1]
"h0": False,                  # Solar Elevation Angle [degrees]
"h0ref": False,               # Solar Elevation Angle Corrected For Refraction [degrees]
"m": False,                   # Relative Optical Airmass [-]
"rotm": False,                # Rayleigh Optical Thickness At Airmass m [-]
"Tl2": False,                # Airmass 2 Linke Athmospheric Turbidity Factor [-]
"B0c": False,                 # Beam Irradiance Normal To The Solar Beam [W m-2]
"Bhc": False,                 # Beam Irradiance At A Horizontal Surface [W m-2]
"Dhc": False,                 # Diffuse Irradiance At A Horizontal Surface [W/m2]
"ra_hor_clear_i": False,      # Total Clear-sky Irradiance On A Horizontal Surface
"emiss_atm_i": False,        # Instantaneous Athmospheric Emissivity [-]
"rn_bare": False,             # Net Radiation Bare Soil [W m-2]
"rn_full": False,             # Net Radiation Full Vegetation [W m-2]
"u_b_i_full": False,          # Instantaneous Wind Speed At Blending Height For Full Vegetation [m s-1]
"u_star_i_bare": False,       # Instantaneous Friction Velocity Bare Soil [m s-1]
"u_star_i_full": False,       # Instantaneous Friction Velocity Vegetation [m s-1]
"u_i_soil": False,            # Instantaneous Wind Speed Just Above Soil Surface [m s-1]
"ras": False,                # Aerodynamical Resistance [s m-1]
"raa": False,                 # Aerodynamical Resistance [s m-1]
"rac": False,                 # Aerodynamical Resistance Canopy [s m-1]        
"t_max_bare": False,          # Maximum Temperature At Bare Soil [K]
"t_max_full": False,          # Maximum Temperature At Full Dry Vegetation [K]
"t_min_bare": False,          # Minimum Temperature At Bare Soil [K]
"t_min_full": False,          # Minimum Temperature At Full Dry Vegetation [K]
"w_i": False,
"t_dew_i": False,             # Instantaneous Dew Point Temperature [C]
"t_wet_i": False,             # Instantaneous Wet Bulb Temperature [C]
"t_wet_k_i": False,           # Instantaneous Wet Bulb Temperature [K]
"lst_max": False,            # Maximum Temperature At Dry Conditions [K]
"lst_min": False,            # Minimum Temperature At Dry Conditions [K]
"se_root": True,             # Relative Root Zone Soil Moisture Based On LST [%] 
"stress_moist": False,        # Stress Factor For Root Zone Moisture [-]        
"r_canopy_0": False,          # Atmospheric Canopy Resistance [s m-1]
"r_canopy": False,            # Canopy Resistance [s m-1]
"z_obst": False,              # Obstacle Height [m]
"z_oro": False,               # Orographic Roughness [m]
"z0m": False,                 # Roughness Length [m]
"ra_canopy_init": False,      # Canopy Resistance Without Stability Corrections [s m-1]
"u_b_24": False,              # Daily Wind Speed At Blending Height [m s-1]
"disp": False,                # Displacement Height [m]
"u_star_24_init": False,      # Initial Estimate Of The Daily Friction Velocity [m s-1] 
"ad_dry_24": False,           # Daily Dry Air Density [kg m-3]
"ad_moist_24": False,         # Daily Moist Air Desity [kg m-3]
"ad_24": False,               # Daily Air Density [kg m-3]
"psy_24": False,              # Daily Psychrometric Constant [mbar K-1]
"ssvp_24": False,             # Daily Slope Of Saturated Vapour Pressure Curve [mbar K-1]
"t_24_init": False,           # Initial Estimate Radiation Equivalent Daily Transpiration [W m-2]
"h_canopy_24_init": False,    # Initial Estimate Of Sensible Heat Flux
"t_24": False,                # Daily Transpiration Energy Equivalent [W m-2]
"t_24_mm": True,             # Daily Transpiration [mm day-1]
"tpot_24": False,     
"tpot_24_mm": False,
"sf_soil": False,             # Soil Fraction [-]
"rn_24_soil": False,          # Net Radiation For The Soil [W m-2]
"r_soil": False,              # Soil Resistance [s m-1]
"ra_soil_init": False,        # Aerodynamic Resistance Withou Stability Corrections [s m-1]
"u_b_24": False,              # Daily Wind Speed At Blending Height [m]
"u_star_24_soil_init": False, # Initial Estimate Of The Daily Friction Velocity For Soil [m s-1]
"g0_bs": False,               # Bare Soil Heat Flux [W m-2]
"g0_24": False,               # Daily Soil Heat Flux G [W m-2]
"e_24_init": False,           # Initial Estimate Radiation Equivalent Daily Evaporation [W m-2]
"h_soil_24_init": False,      # Initial Estimate Of The Sensible Heat Flux For Soil [W m-2]
"e_24": False,                # Daily Evaporation Energy Equivalent [W m-2]
"e_24_mm": True,             # Daily Evaporation [mm day-1]
"et_24_mm": True,            # Daily Evapotranspiration [mm day-1]
"rn_24_grass": False,         # Daily Net Radiation For Reference Grass [W m-2]
"et_ref_24": False,           # Reference Evapotranspiration envergy equivalent [W m-2]
"et_ref_24_mm": True,        # Reference Evapotranspiration [mm day-1]
"biomass_prod": True,        # Biomass Production [kg/ha]
"L_bare": False,
"L_full": False,
"trans_24": False,
}